#include "FxCyanF.hpp"

// C++ system level
#include <string>
#include <cstdio>         // snprintf
// #include <cstdlib>
// #include <algorithm>
// #include <functional>

// ESP32 specific
#include "esp_log.h"

// misc
// #include <nlohmann/json.hpp>

// qthing stuff
// #include <qthing>
// #include <qthing/mqtt_common.hpp>

// project specific
#include "NVSExplorer.hpp"
#include "SpiderLib/NumberTypes.hpp"
// #include "FxVSync.hpp"
// #include "CyanBus.hpp"
// #include "Metrics.hpp"



namespace SiliconTorch {

  namespace FxCyanF {



    bool configureFromNVS(FxCyanF& target) {

      if (target.getChannelCount() > 0) {
        ESP_LOGW(TAG, "Target mustn't have any channels configured");
        return false;
      }


      std::string ns = "fxCyan";

      u32 frq = 0;
      u8 res = 0;

      frq = NVSExplorer::NVSExplorer::instance().getUnsignedInt(ns, "frequency", frq);
      res = NVSExplorer::NVSExplorer::instance().getUnsignedInt(ns, "resolution", res);

      if (frq == 0 || res == 0) {
        ESP_LOGW(TAG, "No config for CyanLight found");  // TODO: Log-Level Debug
        return false;
      }


      ESP_LOGW(TAG, "Found config for FxCyanF{ frequency[ %d Hz ]  resolution[ %d Bit ] } in NVS", frq, res);  // TODO: Log-Level Debug

      for (u8 ch = 0; ch < MAX_CHANNELS; ch++) {

        i8 gpio = -1;
        
        char channel[32];
        snprintf(channel, 32, "ch%d_gpio", ch);

        //nvs_get_i8(nvs, channel, &gpio);

        if (gpio >= 0) {

          target.addChannel((u8)gpio);

          ESP_LOGW(TAG, "channel[ %d ]  gpio[ %d ]", ch, gpio);  // TODO: Log-Level Info

        } else {
          break;
        }
      }

      return true;
    }


  }
}
